#!/usr/bin/perl

# path where the routines reside, 
# IMPORTANT: this should be changed to the src directory on your machine
$path = "/home/jpei/sd/src/";

# blastpgp program 
# IMPORTANT: this should be changed to the blastpgp program on your machine
$blastpgp = "/data/usr1/seals/bin/blastpgp";

# blast parameters
$thr = 0.01;
$database = "nr";

# design parameters: temperature and weight
$T = 0.13;
$w = 0.90;
$run = 70;
$seed = -1;

# routines to be used
$getSeq = $path."getSeq";
$seqcollection = $path."seqcollection";
$opw = $path."opw.perl";
unless( -e $getSeq)  { die "file not exist: getSeq in $path\n"; }
unless( -e $seqcollection) {die "file not exist: seqcollection in $path\n"; }
unless( -e $opw) { die "file not exist: opw in $path\n"; }
	
# program usage information
if($#ARGV <=  0) {
   printUsage();
}

# -w option
$i = 0;
while($i <$#ARGV) {
   if($ARGV[$i] eq "-w") {
        $w = $ARGV[$i+1];
        $i++; next;
    }

    $i++;
}

if( ($w>1) || ($w<0) ) {die "weight must be between 0 and 1";}

# directory name and the pdb file to work on
use Cwd;
$root_dir = getcwd."/";
$pdb = $ARGV[0];
$pdb =~ s/\..*$//;
$pdbfile = $root_dir.$pdb.".pdb";
unless(-e $pdbfile) { die "pdbfile doesnot exist: $pdbfile\n"; }


$log = $root_dir.$pdb.".design".".log";
#print $log, "\n";
open(LOGFILE, "> $log") || die "cannot open the log file\n";
print LOGFILE "\n\n", (scalar localtime)."\t"."a new round of design begins", "\n";
close(LOGFILE);


# read the pdbfile, output the sequence as the fasta file, copy the fastafile 
# to the directory of blast
# move the alignment file to the directory of blast

chdir($root_dir);
system("$getSeq $pdbfile");
$blast_dir = $root_dir."blast";
unless(-e $blast_dir) { system("mkdir $blast_dir"); }
$fasta_file = $pdb.".fa";
unless(-e $fasta_file) {die "$fasta_file doesnot exit\n";}
open(FA, $fasta_file) || die "cannot open the $fasta_file\n";
$step = 0;
while(<FA>) {
  if(/>/) {next;}
  chomp;
  $step = length($_); $alilen = $step;
  last;
}
close(FA);
#print $step."\n";
system("mv $fasta_file $blast_dir");
system("cp $ARGV[1] $blast_dir");

# run blast for 6 iterations, output the matrix and the alignment
# parse the matrix and the alignment

chdir($blast_dir);
$mat1_file = $pdb.".mat1";
$mat_file = $pdb.".mat";
$aln_file1 = $pdb.".aln1";
$aln_file0 = $pdb.".aln0";
$aln_file = $pdb.".aln";
system("cp $ARGV[1] $aln_file 2>/dev/null");
$run = `wc -l $aln_file` + 0;
$out_file = $pdb.".br";

if(-e $mat1_file) {`rm $mat1_file`;}

`$blastpgp -i $fasta_file -d $database -e $thr -h $thr -j 1 -o $out_file -I T -Q $mat1_file -b 5000 -v 5000 -B $aln_file 2>blast.log`;

unless (-e $mat1_file) {
	print "something is wrong with blastpgp", "\n";
	print $mat1_file, " is not generated", "\n";
	print "see blast.log for error message", "\n";
	exit(0);
}

#print "$blastpgp -i $fasta_file -d $database -e $thr -h $thr -j 1 -o $out_file -a 6 -I T -Q $mat1_file -b 5000 -v 5000 -B $aln_file 2>/dev/null";

system("cut -c1-70 $mat1_file| perl -pe 's/^\s*[LKSP].*//' |grep -v Lambda | grep '[0-9a-zA-Z]' > $mat_file");

# set up a directory named "INPUT", which contains the following files:
#  1. multiple sequence alignment (.aln)
#  2. the pdb file (.pdb)
#  3. the PSSM file (.mat)
#  4. averagescore for the single residue term
#  5. M-J potential matrix (mj98)
#  6. contact frequencies in decoy model (myfreq.txt)
#  7. amino acid concentrations (wwconc, not used)

open(LOGFILE, ">> $log")|| die "cannot open the log file\n";
print LOGFILE (scalar localtime)."\t"."Start setting up the directory of INPUT\n";
close(LOGFILE);

$common_dir = $path;
chdir($root_dir);
$input_dir = $root_dir."INPUT";
unless(-e $input_dir) {system("mkdir $input_dir");}
system("cp $pdbfile $input_dir");
system("cp blast/$aln_file $input_dir");
system("cp blast/$mat_file $input_dir");
system("cp $common_dir/mj98 $input_dir");
system("cp $common_dir/averagescore $input_dir");
system("cp $common_dir/myfreq.txt $input_dir");
system("cp $common_dir/wwconc $input_dir");

open(LOGFILE, ">> $log")|| die "cannot open the log file\n";
print LOGFILE (scalar localtime)."\t"."finish setting up INPUT\n";
close(LOGFILE);


# optimize the weight here
open(LOGFILE, ">> $log")|| die "cannot open the log file\n";
print LOGFILE (scalar localtime)."\t"."Start optimizing the weight\n";
close(LOGFILE);
use IO::Handle;
if($pid_opw = open(CHILD, "-|")) {
   chomp($w1 = <CHILD>);
   $w1 = $w1 + 0;
   close(CHILD);
}
else {
   die "cannot fork: $!" unless defined $pid_opw;
   STDOUT->autoflush(1);
   $w2 = `$opw $pdb` +0;
   print STDOUT $w2;
   exit;
}
#if($pid_opw = fork() ) {
#;}
#else {
#   $w1 = `$opw $pdb` +0;
#   print "w1: ", $w1, "\n";
#}

#use IO::Handle;
#pipe(READER, WRITER);
#WRITER->autoflush(1);
#chdir($root_dir);
#if( $pid_opw = fork() ) {
#    close WRITER;
#    chomp($w1 = <READER>);
#    print $w1."\n";
#    close READER;
#    waitpid($pid_opw, 0);
#}
#else {
#    die "cannot fork: $!" unless defined $pid_opw;
#    close READER;
#    $w_temp = `$opw $pdb`;
#    #$w_temp = 0.85;
#    
#    printf WRITER "%3.2f",$w_temp;
#    close WRITER;
#    exit;
#}

open(LOGFILE, ">> $log")|| die "cannot open the log file\n";
print LOGFILE (scalar localtime)."\t"."weight optimized:\t", $w1, "\n";
close(LOGFILE);

# set up a directory named "design", which contains the configure file 
# "configurefile" for generating designed sequences.  set up a directory
# named "homology***", "***" stands for the weight, for blast runs.   

open(LOGFILE, ">> $log")|| die "cannot open the log file\n";
print LOGFILE (scalar localtime)."\t"."Start setting up directories design and homology\n";
close(LOGFILE);

$design_dir = $root_dir."design";
unless(-e $design_dir) {system("mkdir $design_dir");}
chdir($design_dir);
system("cp $seqcollection .");
open(CF, ">configurefile") || die "cannot open configurefile for write\n";
print CF "PDB_FILE "."../INPUT/".$pdb.".pdb"."\n!\n";
print CF "THR_CONTACT 6.5"."\n!\n";
print CF "AMINO_TYPES 20"."\n!\n";
print CF "MC_TEMP ".$T."\n!\n";
print CF "AVERAGE_SCORE ../INPUT/averagescore"."\n!\n";
print CF "NRUN ".$run."\n!\n";
print CF "STEP ".$step."\n!\n";
print CF "EMFILE ../INPUT/mj98"."\n!\n";
print CF "COMPOSITIONFILE ../INPUT/wwconc"."\n!\n";
print CF "COMPOSITION_VARIATION 100"."\n!\n";
print CF "CONTACT_FREQ ../INPUT/myfreq.txt"."\n!\n";
print CF "OUT_FILE ../INPUT/junk"."\n!\n";
print CF "PSSM ../INPUT/".$pdb.".mat"."\n!\n";
print CF "NATIVE_ALN ../INPUT/".$pdb.".aln"."\n!\n";
#print CF "NATIVE_ALN ".$pdb.".aln"."\n!\n";

# wait for the optimized weight
waitpid($pid_opw, 0);
print "optimized weight: ", $w1, "\n";
if( ($w1>0.98)||($w1<0.8) ) {
    #print "Warning: abnormal weight: $w1\n";
    open(LOGFILE, ">>$log")|| die "cannot open the log file\n";
    print LOGFILE "Warning: abnormal weight: $w1\n";
    print LOGFILE " weight using default: 0.9\n";
    close(LOGFILE);
    #print $w1;
    #exit;
}

else { 
    $w = $w1;
}

print CF "RW ".$w."\n!\n";
print CF "SEED -1"."\n!\n";
print CF "ALL 0"."\n!\n";
print CF "DIR ".$homology_dir."/"."\n!\n";
print CF "TARGET ".$pdb."\n!\n";
close(CF);

open(LOGFILE, ">>$log")|| die "cannot open the log file\n";
print LOGFILE (scalar localtime)."\t"."finishing the configurefile\n";
close(LOGFILE);

open(LOGFILE, ">>$log")|| die "cannot open the log file\n";
print LOGFILE (scalar localtime)."\t"."Start designing sequences\n";
close(LOGFILE);

# BEGIN ##################################
# parse the command line

$i = 0;
while($i <$#ARGV) {
    #print "1";

    if($ARGV[$i] eq "-step") {
        $step = $ARGV[$i+1];
        #print "step: ".$step."\n";
        $i++;  next;
    }

    if($ARGV[$i] eq "-run") {
        $run = $ARGV[$i+1];
        #print "run: ".$run."\n";
        $i++; next;
    }

    if($ARGV[$i] eq "-T") {
        $T = $ARGV[$i+1];
        $i++; next;
    }

    if($ARGV[$i] eq "-seed") {
        $seed = $ARGV[$i+1];
        $i++; next;
    }

    if($ARGV[$i] eq "-w") {
        $w = $ARGV[$i+1];
        $i++; next;
    }
	
    if($ARGV[$i] eq "-d") {
	$step = int $ARGV[$i+1] * $alilen;
        $i++; next;
    }

    $i++;

}

#print "step: ".$step."\n";
#print "run: ".$run."\n";
#print "T: ".$T."\n";
#print "seed: ".$seed."\n";
#print "w: ".$w."\n";

# end parse the command line
# END #########################################

$design_ali =  "ac_step".$step.".aln";
`seqcollection configurefile -step $step -run $run -T $T -w $w -seed $seed > $design_ali`;
$design_ali1 = $design_ali."1";
`grep design $design_ali > $design_ali1`;
`grep -v design $design_ali > info`;
`mv $design_ali1 $design_ali`;
`cp $design_ali ../`;

print "designed sequences in file ", $design_ali, "\n";
print "program finished", "\n";

sub printUsage {
    print "design.perl: design sequences using a structure and an alignment\n";
    print "Usage: design.perl pdbfile alnfile (options)\n";
    print "Options: \n";
    print "         -step: number of accepted steps\n";
    print "         -d: the evolutionary distance\n";
    print "         -run: number of designed sequences\n";
    print "         -T: temperature\n";
    print "         -seed: for random number\n";
    print "         -w: weight\n";
}
