#ifndef _EMATRIX_H_
#define _EMATRIX_H_

#include <iostream>
#include <fstream>


class ematrix{
  char efile[100];
  ifstream in;
  int Nabc;

 public:
  char ABC20[21]; //  20 amino acids + 1 empty space, designated by '.'
  char ABC6[7]; // 6 letter code + 1 empty space, designated by '.'
  double **e;
  double **e2;
  double **e6;
  double **e62;


  ematrix(char *fname, int n){
    int i,j;
    char abc20[21] = {'C','M','F','I','L','V','W','Y','A','G',\
		      'T','S','Q','N','E','D','H','R','K','P','.'};
    for(i=0;i<21;i++)ABC20[i]=abc20[i];
    char abc6[7] = {'l','r','p','+','-','s','.'};
    // 'l' - aliphatic = {AVLIMC}
    // 'r' - aromatic  = {FWYH}
    // 'p' - polar     = {STNQ}
    // '+' - positive  = {KR}
    // '-' - negative  = {DE}
    // 's' - special   = {GP}
    for(i=0;i<7;i++)ABC6[i]=abc6[i];

    strcpy(efile,fname);
    Nabc=n;
    // READING ENERGY MATRIX
    in.open(efile,ios::in);
    if(!in)error("Cannot open file with Energy Matrix");

    e = new double*[Nabc+1];
    for(i=0;i<=Nabc;i++) e[i] = new double[Nabc+1]; 
    e2 = new double*[Nabc+1];
    for(i=0;i<=Nabc;i++) e2[i] = new double[Nabc+1]; 
    e6 = new double*[7];
    for(i=0;i<=6;i++) e6[i] = new double[7]; 
    e62 = new double*[7];
    for(i=0;i<=6;i++) e62[i] = new double[7]; 

    /*for(i=0;i<Nabc;i++){
      for(j=0;j<=i;j++){
	in>>e[j][i];
	e[i][j]=e[j][i];
	e2[j][i]=e[j][i]*e[j][i];
	e2[i][j]=e2[j][i];
      }
    }*/

    for(i=0;i<Nabc;i++) {
	for(j=0;j<Nabc;j++) {
	   in >> e[i][j];
	   e2[i][j]=e[i][j]*e[i][j];
	}
    }
    in.close();
    //have to account for the empty space '.'
    for(i=0;i<=Nabc;i++)e[20][i]=e[i][20]=e2[20][i]=e2[i][20]=0; 
    // END READING ENERGY MATRIX

    // INITIALIZING 6-LETTER ENERGY MATRIX
    short num_el[7]={6,4,4,2,2,2,1};
    int pos_el[7][6]={0,1,3,4,5,8,\
			2,6,7,16,20,20,\
			10,11,12,13,20,20,\
			17,18,20,20,20,20,\
			14,15,20,20,20,20,\
			9,19,20,20,20,20,\
			20,20,20,20,20,20};

    int k,l;
    double count;
    for(i=0;i<7;i++){
      for(j=i;j<7;j++){
	e6[i][j]=0;
	count=0;
	for(k=0;k<num_el[i];k++){
	  for(l=0;l<num_el[j];l++){
	    e6[i][j]+=e[pos_el[i][k]][pos_el[j][l]];
	    count+=1.0;
//cout<<i<<"\t"<<j<<"\t"<<pos_el[i][k]<<"\t"<<pos_el[j][l]<<"\t"<<e[pos_el[i][k]][pos_el[j][l]]<<"\t"<<endl<<flush;
	  }
	}
	e6[i][j]/=count;
//cout<<"-------------------------------\n"<<ABC6[i]<<"\t"<<ABC6[j]<<"\t"<<e6[i][j]<<"\t"<<count<<"\n-------------------------------"<<endl<<flush;
	e6[j][i]=e6[i][j];
	e62[i][j]=e62[j][i]=(e6[i][j]*e6[i][j]);
      }
    }
    for(i=0;i<=6;i++)e6[6][i]=e6[i][6]=e62[6][i]=e62[i][6]=0;
    // END INITIALIZING 6-LETTER ENERGY MATRIX

  };

  ~ematrix(){
    int i;
    for(i=0;i<=Nabc;i++)delete [] e[i];
    delete [] e;
    for(i=0;i<=Nabc;i++)delete [] e2[i];
    delete [] e2;
    for(i=0;i<=6;i++)delete [] e6[i];
    delete [] e6;
    for(i=0;i<=6;i++)delete [] e62[i];
    delete [] e62;
  };

  short ind(char lett){
    for(int i=0;i<21;i++)if(lett==ABC20[i])return i;
  };

  short ind6(char lett){
    if((lett=='A')||(lett=='V')||(lett=='L')||(lett=='I')||\
       (lett=='M')||(lett=='C'))return 0;
    if((lett=='F')||(lett=='W')||(lett=='Y')||(lett=='H'))return 1;
    if((lett=='S')||(lett=='T')||(lett=='N')||(lett=='Q'))return 2;
    if((lett=='K')||(lett=='R'))return 3;
    if((lett=='D')||(lett=='E'))return 4;
    if((lett=='G')||(lett=='P'))return 5;
    if((lett=='.'))return 6;
  };

  short to6(short id){
    if((id==8)||(id==5)||(id==4)||(id==3)||(id==1)||(id==0))return 0;
    if((id==2)||(id==6)||(id==7)||(id==16))return 1;
    if((id==11)||(id==10)||(id==13)||(id==12))return 2;
    if((id==18)||(id==17))return 3;
    if((id==15)||(id==14))return 4;
    if((id==9)||(id==19))return 5;
    if((id==20))return 6;
  };



  double ave(void){
    double av=0;
    for(int i=0;i<Nabc;i++){
      for(int j=0;j<Nabc;j++){
	av+=e[i][j];
      }
    }
    av/=((double)(Nabc*Nabc));

    return av;
  };

  double sigma2(void){
    double s=0;
    for(int i=0;i<Nabc;i++){
      for(int j=0;j<Nabc;j++){
	s+=e2[i][j];
      }
    }
    s/=((double)(Nabc*Nabc));
    s-=(ave()*ave());

    return s;
  };

 private:
  void error(char *msg){
    cerr<<msg<<endl<<flush;
    exit(1);
  }   

};

#endif /* _EMATRIX_H_ */
